import logging

from flask import Blueprint
from flask import flash
from flask import redirect
from flask import render_template
from flask import request
from flask_login import current_user
from flask_login import login_required
from pydantic import ValidationError
from sqlalchemy import and_
from sqlalchemy import or_
from sqlalchemy import select

from htmx_contact import Session
from htmx_contact.models import Contact
from htmx_contact.validators import NewContactValidator
from htmx_contact.validators import repack_validation_errors

logger = logging.getLogger(__name__)

bp = Blueprint("main", __name__, url_prefix="/")


@bp.route("/", methods=["GET"])
def index():
    return redirect("/contacts")


@bp.route("/contacts", methods=["GET"])
@login_required
def contacts():
    search = request.args.get("q")
    if search is not None:
        stmt = (
            select(Contact)
            .where(
                and_(
                    Contact.user_id == current_user.id,
                    or_(
                        Contact.first_name.like(f"%{search}%"),
                        Contact.last_name.like(f"%{search}%"),
                        Contact.email.like(f"%{search}%"),
                    ),
                )
            )
            .order_by(Contact.last_name, Contact.first_name)
        )
    else:
        stmt = select(Contact).where(Contact.user_id == current_user.id).order_by(Contact.last_name, Contact.first_name)

    with Session() as session:
        results = session.scalars(stmt).all()

    return render_template("contacts.html", contacts=results)


@bp.route("/contacts/new", methods=["GET", "POST"])
def new_contact():
    if request.method == "POST":
        try:
            validated_form = NewContactValidator(
                # first_name=request.form.get("first_name"),
                # last_name=request.form.get("last_name"),
                # phone=request.form.get("phone"),
                # email=request.form.get("email"),
                **request.form
            )
            with Session.begin() as session:
                contact = Contact(user_id=current_user.id, **validated_form.model_dump())
                session.add(contact)
                session.commit()
            flash("New Contacted added!")
            return redirect("/contacts")
        except ValidationError as error:
            logger.info("validation error: %r", error)
            return render_template("new_contact.html", errors=repack_validation_errors(error.errors()))
    return render_template("new_contact.html", errors={})
